@@ -1,12 +1,13 @@
 from optparse import make_option
 
 from django.conf import settings
-from django.core.management.commands.runserver import BaseRunserverCommand
+from django.core.management.commands.runserver import Command as RunserverCommand
 
 from django.contrib.staticfiles.handlers import StaticFilesHandler
 
-class Command(BaseRunserverCommand):
-    option_list = BaseRunserverCommand.option_list + (
+
+class Command(RunserverCommand):
+    option_list = RunserverCommand.option_list + (
         make_option('--nostatic', action="store_false", dest='use_static_handler', default=True,
             help='Tells Django to NOT automatically serve static files at STATIC_URL.'),
         make_option('--insecure', action="store_true", dest='insecure_serving', default=False,
