@@ -98,13 +98,11 @@ class ModelBase(type):
                 msg = (
                     "Model class %s.%s doesn't declare an explicit app_label "
                     "and either isn't in an application in INSTALLED_APPS or "
-                    "else was imported before its application was loaded. " %
+                    "else was imported before its application was loaded. "
+                    "This will no longer be supported in Django 1.9." %
                     (module, name))
-                if abstract:
-                    msg += "Its app_label will be set to None in Django 1.9."
-                else:
-                    msg += "This will no longer be supported in Django 1.9."
-                warnings.warn(msg, RemovedInDjango19Warning, stacklevel=2)
+                if not abstract:
+                    warnings.warn(msg, RemovedInDjango19Warning, stacklevel=2)
 
                 model_module = sys.modules[new_class.__module__]
                 package_components = model_module.__name__.split('.')
@@ -1461,6 +1459,8 @@ def model_unpickle(model_id, attrs, factory):
     Used to unpickle Model subclasses with deferred fields.
     """
     if isinstance(model_id, tuple):
+        if not apps.ready:
+            apps.populate(settings.INSTALLED_APPS)
         model = apps.get_model(*model_id)
     else:
         # Backwards compat - the model was cached directly in earlier versions.
