@@ -4,12 +4,14 @@ Classes to represent the default SQL aggregate functions
 import copy
 
 from django.db.models.fields import IntegerField, FloatField
+from django.db.models.lookups import RegisterLookupMixin
+from django.utils.functional import cached_property
 
-# Fake fields used to identify aggregate types in data-conversion operations.
-ordinal_aggregate_field = IntegerField()
-computed_aggregate_field = FloatField()
 
-class Aggregate(object):
+__all__ = ['Aggregate', 'Avg', 'Count', 'Max', 'Min', 'StdDev', 'Sum', 'Variance']
+
+
+class Aggregate(RegisterLookupMixin):
     """
     Default SQL Aggregate.
     """
@@ -55,14 +57,23 @@ class Aggregate(object):
 
         while tmp and isinstance(tmp, Aggregate):
             if getattr(tmp, 'is_ordinal', False):
-                tmp = ordinal_aggregate_field
+                tmp = self._ordinal_aggregate_field
             elif getattr(tmp, 'is_computed', False):
-                tmp = computed_aggregate_field
+                tmp = self._computed_aggregate_field
             else:
                 tmp = tmp.source
 
         self.field = tmp
 
+    # Two fake fields used to identify aggregate types in data-conversion operations.
+    @cached_property
+    def _ordinal_aggregate_field(self):
+        return IntegerField()
+
+    @cached_property
+    def _computed_aggregate_field(self):
+        return FloatField()
+
     def relabeled_clone(self, change_map):
         clone = copy.copy(self)
         if isinstance(self.col, (list, tuple)):
@@ -76,9 +87,9 @@ class Aggregate(object):
         if hasattr(self.col, 'as_sql'):
             field_name, params = self.col.as_sql(qn, connection)
         elif isinstance(self.col, (list, tuple)):
-            field_name = '.'.join([qn(c) for c in self.col])
+            field_name = '.'.join(qn(c) for c in self.col)
         else:
-            field_name = self.col
+            field_name = qn(self.col)
 
         substitutions = {
             'function': self.sql_function,
@@ -88,11 +99,19 @@ class Aggregate(object):
 
         return self.sql_template % substitutions, params
 
+    def get_group_by_cols(self):
+        return []
+
+    @property
+    def output_field(self):
+        return self.field
+
 
 class Avg(Aggregate):
     is_computed = True
     sql_function = 'AVG'
 
+
 class Count(Aggregate):
     is_ordinal = True
     sql_function = 'COUNT'
@@ -101,12 +120,15 @@ class Count(Aggregate):
     def __init__(self, col, distinct=False, **extra):
         super(Count, self).__init__(col, distinct='DISTINCT ' if distinct else '', **extra)
 
+
 class Max(Aggregate):
     sql_function = 'MAX'
 
+
 class Min(Aggregate):
     sql_function = 'MIN'
 
+
 class StdDev(Aggregate):
     is_computed = True
 
@@ -114,9 +136,11 @@ class StdDev(Aggregate):
         super(StdDev, self).__init__(col, **extra)
         self.sql_function = 'STDDEV_SAMP' if sample else 'STDDEV_POP'
 
+
 class Sum(Aggregate):
     sql_function = 'SUM'
 
+
 class Variance(Aggregate):
     is_computed = True
 
