@@ -97,14 +97,16 @@ class TranslateNode(Node):
 
 
 class BlockTranslateNode(Node):
+
     def __init__(self, extra_context, singular, plural=None, countervar=None,
-            counter=None, message_context=None):
+            counter=None, message_context=None, trimmed=False):
         self.extra_context = extra_context
         self.singular = singular
         self.plural = plural
         self.countervar = countervar
         self.counter = counter
         self.message_context = message_context
+        self.trimmed = trimmed
 
     def render_token_list(self, tokens):
         result = []
@@ -115,7 +117,10 @@ class BlockTranslateNode(Node):
             elif token.token_type == TOKEN_VAR:
                 result.append('%%(%s)s' % token.contents)
                 vars.append(token.contents)
-        return ''.join(result), vars
+        msg = ''.join(result)
+        if self.trimmed:
+            msg = translation.trim_whitespace(msg)
+        return msg, vars
 
     def render(self, context, nested=False):
         if self.message_context:
@@ -145,9 +150,15 @@ class BlockTranslateNode(Node):
             else:
                 result = translation.ugettext(singular)
         default_value = settings.TEMPLATE_STRING_IF_INVALID
-        render_value = lambda v: render_value_in_context(
-            context.get(v, default_value), context)
-        data = dict([(v, render_value(v)) for v in vars])
+
+        def render_value(key):
+            if key in context:
+                val = context[key]
+            else:
+                val = default_value % key if '%s' in default_value else default_value
+            return render_value_in_context(val, context)
+
+        data = dict((v, render_value(v)) for v in vars)
         context.pop()
         try:
             result = result % data
@@ -195,6 +206,7 @@ def do_get_available_languages(parser, token):
         raise TemplateSyntaxError("'get_available_languages' requires 'as variable' (got %r)" % args)
     return GetAvailableLanguagesNode(args[2])
 
+
 @register.tag("get_language_info")
 def do_get_language_info(parser, token):
     """
@@ -214,6 +226,7 @@ def do_get_language_info(parser, token):
         raise TemplateSyntaxError("'%s' requires 'for string as variable' (got %r)" % (args[0], args[1:]))
     return GetLanguageInfoNode(parser.compile_filter(args[2]), args[4])
 
+
 @register.tag("get_language_info_list")
 def do_get_language_info_list(parser, token):
     """
@@ -237,18 +250,22 @@ def do_get_language_info_list(parser, token):
         raise TemplateSyntaxError("'%s' requires 'for sequence as variable' (got %r)" % (args[0], args[1:]))
     return GetLanguageInfoListNode(parser.compile_filter(args[2]), args[4])
 
+
 @register.filter
 def language_name(lang_code):
     return translation.get_language_info(lang_code)['name']
 
+
 @register.filter
 def language_name_local(lang_code):
     return translation.get_language_info(lang_code)['name_local']
 
+
 @register.filter
 def language_bidi(lang_code):
     return translation.get_language_info(lang_code)['bidi']
 
+
 @register.tag("get_current_language")
 def do_get_current_language(parser, token):
     """
@@ -268,6 +285,7 @@ def do_get_current_language(parser, token):
         raise TemplateSyntaxError("'get_current_language' requires 'as variable' (got %r)" % args)
     return GetCurrentLanguageNode(args[2])
 
+
 @register.tag("get_current_language_bidi")
 def do_get_current_language_bidi(parser, token):
     """
@@ -287,6 +305,7 @@ def do_get_current_language_bidi(parser, token):
         raise TemplateSyntaxError("'get_current_language_bidi' requires 'as variable' (got %r)" % args)
     return GetCurrentLanguageBidiNode(args[2])
 
+
 @register.tag("trans")
 def do_translate(parser, token):
     """
@@ -333,7 +352,7 @@ def do_translate(parser, token):
         def top(self):
             value = self.value()
 
-            # Backwards Compatiblity fix:
+            # Backwards Compatibility fix:
             # FilterExpression does not support single-quoted strings,
             # so we make a cheap localized fix in order to maintain
             # backwards compatibility with existing uses of ``trans``
@@ -341,9 +360,9 @@ def do_translate(parser, token):
             if value[0] == "'":
                 m = re.match("^'([^']+)'(\|.*$)", value)
                 if m:
-                    value = '"%s"%s' % (m.group(1).replace('"','\\"'), m.group(2))
+                    value = '"%s"%s' % (m.group(1).replace('"', '\\"'), m.group(2))
                 elif value[-1] == "'":
-                    value = '"%s"' % value[1:-1].replace('"','\\"')
+                    value = '"%s"' % value[1:-1].replace('"', '\\"')
 
             noop = False
             asvar = None
@@ -359,13 +378,14 @@ def do_translate(parser, token):
                     asvar = self.tag()
                 else:
                     raise TemplateSyntaxError(
-                        "Only options for 'trans' are 'noop', " \
+                        "Only options for 'trans' are 'noop', "
                         "'context \"xxx\"', and 'as VAR'.")
             return value, noop, asvar, message_context
     value, noop, asvar, message_context = TranslateParser(token.contents).top()
     return TranslateNode(parser.compile_filter(value), noop, asvar,
                          message_context)
 
+
 @register.tag("blocktrans")
 def do_block_translate(parser, token):
     """
@@ -429,6 +449,8 @@ def do_block_translate(parser, token):
                     '"context" in %r tag expected '
                     'exactly one argument.') % bits[0]
                 six.reraise(TemplateSyntaxError, TemplateSyntaxError(msg), sys.exc_info()[2])
+        elif option == "trimmed":
+            value = True
         else:
             raise TemplateSyntaxError('Unknown argument for %r tag: %r.' %
                                       (bits[0], option))
@@ -444,6 +466,8 @@ def do_block_translate(parser, token):
         message_context = None
     extra_context = options.get('with', {})
 
+    trimmed = options.get("trimmed", False)
+
     singular = []
     plural = []
     while parser.tokens:
@@ -465,7 +489,8 @@ def do_block_translate(parser, token):
         raise TemplateSyntaxError("'blocktrans' doesn't allow other block tags (seen %r) inside it" % token.contents)
 
     return BlockTranslateNode(extra_context, singular, plural, countervar,
-            counter, message_context)
+                              counter, message_context, trimmed=trimmed)
+
 
 @register.tag
 def language(parser, token):
