@@ -2,11 +2,12 @@
 Common checksum routines.
 """
 
-__all__ = ['luhn',]
+__all__ = ['luhn']
 
 from django.utils import six
 
-LUHN_ODD_LOOKUP = (0, 2, 4, 6, 8, 1, 3, 5, 7, 9) # sum_of_digits(index * 2)
+LUHN_ODD_LOOKUP = (0, 2, 4, 6, 8, 1, 3, 5, 7, 9)  # sum_of_digits(index * 2)
+
 
 def luhn(candidate):
     """
@@ -17,8 +18,8 @@ def luhn(candidate):
     if not isinstance(candidate, six.string_types):
         candidate = str(candidate)
     try:
-        evens = sum([int(c) for c in candidate[-1::-2]])
-        odds = sum([LUHN_ODD_LOOKUP[int(c)] for c in candidate[-2::-2]])
+        evens = sum(int(c) for c in candidate[-1::-2])
+        odds = sum(LUHN_ODD_LOOKUP[int(c)] for c in candidate[-2::-2])
         return ((evens + odds) % 10 == 0)
     except ValueError:  # Raised if an int conversion fails
         return False
