@@ -690,3 +690,18 @@ class Ticket23605B(models.Model):
 
 class Ticket23605C(models.Model):
     field_c0 = models.FloatField()
+
+
+# db_table names have capital letters to ensure they are quoted in queries.
+class Individual(models.Model):
+    alive = models.BooleanField(default=False)
+
+    class Meta:
+        db_table = 'Individual'
+
+
+class RelatedIndividual(models.Model):
+    related = models.ForeignKey(Individual, related_name='related_individual')
+
+    class Meta:
+        db_table = 'RelatedIndividual'
