@@ -1,9 +1,10 @@
 from __future__ import unicode_literals
 
-from django.core.urlresolvers import NoReverseMatch
+from django.core.urlresolvers import NoReverseMatch, reverse_lazy
 from django.contrib.auth.views import logout
 from django.shortcuts import resolve_url
 from django.test import TestCase
+from django.utils import six
 
 from .models import UnimportantThing
 
@@ -55,6 +56,15 @@ class ResolveUrlTests(TestCase):
         resolved_url = resolve_url(logout)
         self.assertEqual('/accounts/logout/', resolved_url)
 
+    def test_lazy_reverse(self):
+        """
+        Tests that passing the result of reverse_lazy is resolved to a real URL
+        string.
+        """
+        resolved_url = resolve_url(reverse_lazy('logout'))
+        self.assertIsInstance(resolved_url, six.text_type)
+        self.assertEqual('/accounts/logout/', resolved_url)
+
     def test_valid_view_name(self):
         """
         Tests that passing a view function to ``resolve_url`` will result in
